@@ -120,6 +120,8 @@ cc_background_panel_dispose (GObject *object)
   g_free (priv->screenshot_path);
   priv->screenshot_path = NULL;
 
+  g_clear_object (&priv->connection);
+
   G_OBJECT_CLASS (cc_background_panel_parent_class)->dispose (object);
 }
 
@@ -228,7 +230,6 @@ update_display_preview (CcBackgroundPanel *panel)
                                    pixbuf,
                                    0, 0);
       cairo_paint (cr);
-      g_object_unref (pixbuf);
     }
 
   cairo_destroy (cr);
@@ -255,6 +256,8 @@ on_screenshot_finished (GObject *source,
   cairo_surface_t *surface;
   cairo_t *cr;
   GVariant *result;
+  int width;
+  int height;
 
   error = NULL;
   result = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source),
@@ -285,12 +288,12 @@ on_screenshot_finished (GObject *source,
       goto out;
     }
 
+  width = gdk_pixbuf_get_width (pixbuf);
+  height = gdk_pixbuf_get_height (pixbuf);
   surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-                                        data->monitor_rect.width, data->monitor_rect.height);
+                                        width, height);
   cr = cairo_create (surface);
-  gdk_cairo_set_source_pixbuf (cr, pixbuf,
-                               data->capture_rect.x - data->monitor_rect.x,
-                               data->capture_rect.y - data->monitor_rect.y);
+  gdk_cairo_set_source_pixbuf (cr, pixbuf, data->capture_rect.x, data->capture_rect.y);
   cairo_paint (cr);
   g_object_unref (pixbuf);
 
@@ -365,7 +368,7 @@ get_screenshot_async (CcBackgroundPanel *panel)
   if (calculate_contiguous_workarea (data)) {
     g_debug ("Capturing only a portion of the screen");
   } else {
-    g_debug ("Capturing the whole monitor");
+    g_debug ("Capturing the whole screen");
     data->whole_monitor = TRUE;
     data->capture_rect = data->monitor_rect;
   }
