@@ -1062,7 +1062,6 @@ on_control_stream_added (GvcMixerControl *control,
             (GVC_IS_MIXER_SOURCE (stream) == FALSE &&
              GVC_IS_MIXER_SINK (stream) == FALSE &&
              gvc_mixer_stream_is_virtual (stream) == FALSE &&
-             gvc_mixer_stream_is_event_stream (stream) == FALSE &&
              g_strcmp0 (app_id, "org.gnome.VolumeControl") != 0 &&
              g_strcmp0 (app_id, "org.PulseAudio.pavucontrol") != 0)) {
                 GtkWidget      *bar;
@@ -1595,7 +1594,7 @@ on_test_speakers_clicked (GvcComboBox *widget,
                                          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                          NULL);
-        gtk_window_set_has_resize_grip (GTK_WINDOW (d), FALSE);
+        gtk_window_set_resizable (GTK_WINDOW (d), FALSE);
 
         g_free (title);
         speaker_test = gvc_speaker_test_new (dialog->priv->mixer_control,
