@@ -72,6 +72,11 @@ struct _CcUserPanelPrivate {
         UmPasswordDialog *password_dialog;
         UmPhotoDialog *photo_dialog;
         UmHistoryDialog *history_dialog;
+
+        gint other_accounts;
+        GtkTreeIter *other_iter;
+
+        UmAccountDialog *account_dialog;
 };
 
 static GtkWidget *
@@ -140,7 +145,7 @@ user_added (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
         GtkTreeIter iter;
         GtkTreeIter dummy;
         GdkPixbuf *pixbuf;
-        gchar *text;
+        gchar *text, *title;
         GtkTreeSelection *selection;
         gint sort_key;
 
@@ -157,6 +162,7 @@ user_added (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
                 sort_key = 1;
         }
         else {
+                d->other_accounts++;
                 sort_key = 3;
         }
         gtk_list_store_append (store, &iter);
@@ -177,6 +183,19 @@ user_added (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
             !gtk_tree_selection_get_selected (selection, &model, &dummy)) {
                 gtk_tree_selection_select_iter (selection, &iter);
         }
+
+        /* Show heading for other accounts if new one have been added. */
+        if (d->other_accounts == 1 && sort_key == 3) {
+                title = g_strdup_printf ("<small><span foreground=\"#555555\">%s</span></small>", _("Other Accounts"));
+                gtk_list_store_append (store, &iter);
+                gtk_list_store_set (store, &iter,
+                                    TITLE_COL, title,
+                                    HEADING_ROW_COL, TRUE,
+                                    SORT_KEY_COL, 2,
+                                    -1);
+                d->other_iter = gtk_tree_iter_copy (&iter);
+                g_free (title);
+        }
 }
 
 static void
@@ -227,6 +246,7 @@ user_removed (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
         GtkListStore *store;
         GtkTreeIter iter, next;
         ActUser *u;
+        gint key;
 
         g_debug ("user removed: %s\n", act_user_get_user_name (user));
         tv = (GtkTreeView *)get_widget (d, "list-treeview");
@@ -235,12 +255,15 @@ user_removed (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
         store = GTK_LIST_STORE (model);
         if (gtk_tree_model_get_iter_first (model, &iter)) {
                 do {
-                        gtk_tree_model_get (model, &iter, USER_COL, &u, -1);
+                        gtk_tree_model_get (model, &iter, USER_COL, &u, SORT_KEY_COL, &key, -1);
 
                         if (u != NULL) {
                                 if (act_user_get_uid (user) == act_user_get_uid (u)) {
                                         if (!get_next_user_row (model, &iter, &next))
                                                 get_previous_user_row (model, &iter, &next);
+                                        if (key == 3) {
+                                                d->other_accounts--;
+                                        }
                                         gtk_list_store_remove (store, &iter);
                                         gtk_tree_selection_select_iter (selection, &next);
                                         g_object_unref (u);
@@ -250,6 +273,13 @@ user_removed (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
                         }
                 } while (gtk_tree_model_iter_next (model, &iter));
         }
+
+        /* Hide heading for other accounts if last one have been removed. */
+        if (d->other_iter != NULL && d->other_accounts == 0 && key == 3) {
+                gtk_list_store_remove (store, d->other_iter);
+                gtk_tree_iter_free (d->other_iter);
+                d->other_iter = NULL;
+        }
 }
 
 static void show_user (ActUser *user, CcUserPanelPrivate *d);
@@ -321,6 +351,7 @@ select_created_user (GObject *object,
         dialog = UM_ACCOUNT_DIALOG (object);
         user = um_account_dialog_finish (dialog, result);
         gtk_widget_destroy (GTK_WIDGET (dialog));
+        d->account_dialog = NULL;
 
         if (user == NULL)
                 return;
@@ -350,10 +381,8 @@ select_created_user (GObject *object,
 static void
 add_user (GtkButton *button, CcUserPanelPrivate *d)
 {
-        UmAccountDialog *dialog;
-
-        dialog = um_account_dialog_new ();
-        um_account_dialog_show (dialog, GTK_WINDOW (gtk_widget_get_toplevel (d->main_box)),
+        d->account_dialog = um_account_dialog_new ();
+        um_account_dialog_show (d->account_dialog, GTK_WINDOW (gtk_widget_get_toplevel (d->main_box)),
                                 d->permission, select_created_user, d);
 }
 
@@ -408,6 +437,11 @@ delete_user_response (GtkWidget         *dialog,
 
         user = get_selected_user (d);
 
+        /* remove autologin */
+        if (act_user_get_automatic_login (user)) {
+                act_user_set_automatic_login (user, FALSE);
+        }
+
         act_user_manager_delete_user_async (d->um,
                                             user,
                                             remove_files,
@@ -595,6 +629,18 @@ get_login_time_text (ActUser *user)
         return text;
 }
 
+static gboolean
+get_autologin_possible (ActUser *user)
+{
+        gboolean locked;
+        gboolean set_password_at_login;
+
+        locked = act_user_get_locked (user);
+        set_password_at_login = (act_user_get_password_mode (user) == ACT_USER_PASSWORD_MODE_SET_AT_LOGIN);
+
+        return !(locked || set_password_at_login);
+}
+
 static void
 show_user (ActUser *user, CcUserPanelPrivate *d)
 {
@@ -635,9 +681,7 @@ show_user (ActUser *user, CcUserPanelPrivate *d)
         g_signal_handlers_block_by_func (widget, autologin_changed, d);
         gtk_switch_set_active (GTK_SWITCH (widget), act_user_get_automatic_login (user));
         g_signal_handlers_unblock_by_func (widget, autologin_changed, d);
-
-        if (act_user_get_locked (user))
-                gtk_widget_set_sensitive (widget, FALSE);
+        gtk_widget_set_sensitive (widget, get_autologin_possible (user));
 
         widget = get_widget (d, "account-language-combo");
         model = um_editable_combo_get_model (UM_EDITABLE_COMBO (widget));
@@ -1039,7 +1083,8 @@ on_permission_changed (GPermission *permission,
         }
 
         widget = get_widget (d, "remove-user-toolbutton");
-        gtk_widget_set_sensitive (widget, is_authorized && !self_selected);
+        gtk_widget_set_sensitive (widget, is_authorized && !self_selected
+                                  && !would_demote_only_admin (user));
         if (is_authorized) {
                 setup_tooltip_with_embedded_icon (widget, _("Delete the selected user account"), NULL, NULL);
         }
@@ -1066,14 +1111,23 @@ on_permission_changed (GPermission *permission,
                 remove_unlock_tooltip (get_widget (d, "autologin-switch"));
 
         } else if (is_authorized && act_user_is_local_account (user)) {
-                um_editable_combo_set_editable (UM_EDITABLE_COMBO (get_widget (d, "account-type-combo")), TRUE);
+                if (would_demote_only_admin (user)) {
+                        um_editable_combo_set_editable (UM_EDITABLE_COMBO (get_widget (d, "account-type-combo")), FALSE);
+                } else {
+                        um_editable_combo_set_editable (UM_EDITABLE_COMBO (get_widget (d, "account-type-combo")), TRUE);
+                }
                 remove_unlock_tooltip (get_widget (d, "account-type-combo"));
-                gtk_widget_set_sensitive (GTK_WIDGET (get_widget (d, "autologin-switch")), TRUE);
+
+                gtk_widget_set_sensitive (GTK_WIDGET (get_widget (d, "autologin-switch")), get_autologin_possible (user));
                 remove_unlock_tooltip (get_widget (d, "autologin-switch"));
         }
         else {
                 um_editable_combo_set_editable (UM_EDITABLE_COMBO (get_widget (d, "account-type-combo")), FALSE);
-                add_unlock_tooltip (get_widget (d, "account-type-combo"));
+                if (would_demote_only_admin (user)) {
+                        remove_unlock_tooltip (get_widget (d, "account-type-combo"));
+                } else {
+                        add_unlock_tooltip (get_widget (d, "account-type-combo"));
+                }
                 gtk_widget_set_sensitive (GTK_WIDGET (get_widget (d, "autologin-switch")), FALSE);
                 add_unlock_tooltip (get_widget (d, "autologin-switch"));
         }
@@ -1119,7 +1173,6 @@ on_permission_changed (GPermission *permission,
         }
 
         um_password_dialog_set_user (d->password_dialog, user);
-        um_password_dialog_set_privileged (d->password_dialog, is_authorized);
 
         g_object_unref (user);
 }
@@ -1236,14 +1289,8 @@ setup_main_window (CcUserPanelPrivate *d)
                             -1);
         g_free (title);
 
-        title = g_strdup_printf ("<small><span foreground=\"#555555\">%s</span></small>", _("Other Accounts"));
-        gtk_list_store_append (store, &iter);
-        gtk_list_store_set (store, &iter,
-                            TITLE_COL, title,
-                            HEADING_ROW_COL, TRUE,
-                            SORT_KEY_COL, 2,
-                            -1);
-        g_free (title);
+        d->other_accounts = 0;
+        d->other_iter = NULL;
 
         column = gtk_tree_view_column_new ();
         cell = gtk_cell_renderer_pixbuf_new ();
@@ -1385,6 +1432,10 @@ cc_user_panel_dispose (GObject *object)
 {
         CcUserPanelPrivate *priv = UM_USER_PANEL (object)->priv;
 
+        if (priv->um) {
+                g_signal_handlers_disconnect_by_data (priv->um, priv);
+                priv->um = NULL;
+        }
         if (priv->builder) {
                 g_object_unref (priv->builder);
                 priv->builder = NULL;
@@ -1401,6 +1452,10 @@ cc_user_panel_dispose (GObject *object)
                 um_history_dialog_free (priv->history_dialog);
                 priv->history_dialog = NULL;
         }
+        if (priv->account_dialog) {
+                gtk_dialog_response (GTK_DIALOG (priv->account_dialog), GTK_RESPONSE_DELETE_EVENT);
+                priv->account_dialog = NULL;
+        }
         if (priv->language_chooser) {
                 gtk_widget_destroy (priv->language_chooser);
                 priv->language_chooser = NULL;
@@ -1409,6 +1464,10 @@ cc_user_panel_dispose (GObject *object)
                 g_object_unref (priv->permission);
                 priv->permission = NULL;
         }
+        if (priv->other_iter) {
+                gtk_tree_iter_free (priv->other_iter);
+                priv->other_iter = NULL;
+        }
         G_OBJECT_CLASS (cc_user_panel_parent_class)->dispose (object);
 }
 
