@@ -284,9 +284,9 @@ set_entry_validation_error (GtkEntry    *entry,
                             const gchar *text)
 {
         g_object_set (entry, "caps-lock-warning", FALSE, NULL);
-        gtk_entry_set_icon_from_icon_name (entry,
-                                           GTK_ENTRY_ICON_SECONDARY,
-                                           "dialog-error-symbolic");
+        gtk_entry_set_icon_from_stock (entry,
+                                       GTK_ENTRY_ICON_SECONDARY,
+                                       GTK_STOCK_CAPS_LOCK_WARNING);
         gtk_entry_set_icon_activatable (entry,
                                         GTK_ENTRY_ICON_SECONDARY,
                                         TRUE);
@@ -542,10 +542,7 @@ is_valid_username (const gchar *username, gchar **tip)
                         *tip = g_strdup (_("The username cannot start with a '-'"));
                 }
                 else {
-                        *tip = g_strdup (_("The username must only consist of:\n"
-                                          " \xe2\x9e\xa3 letters from the English alphabet\n"
-                                          " \xe2\x9e\xa3 digits\n"
-                                          " \xe2\x9e\xa3 any of the characters '.', '-' and '_'"));
+                        *tip = g_strdup (_("The username should only consist of lower and upper case letters from a-z, digits and any of characters '.', '-' and '_'"));
                 }
         }
         else {
@@ -1034,3 +1031,41 @@ set_user_icon_data (ActUser   *user,
 
         g_free (path);
 }
+
+static guint
+get_num_admin (ActUserManager *um)
+{
+        GSList *list;
+        GSList *l;
+        guint num_admin = 0;
+
+        list = act_user_manager_list_users (um);
+        for (l = list; l != NULL; l = l->next) {
+                ActUser *u = l->data;
+                if (act_user_get_account_type (u) == ACT_USER_ACCOUNT_TYPE_ADMINISTRATOR) {
+                        num_admin++;
+                }
+        }
+        g_slist_free (list);
+
+        return num_admin;
+}
+
+gboolean
+would_demote_only_admin (ActUser *user)
+{
+
+        ActUserManager *um = act_user_manager_get_default ();
+
+        /* Prevent the user from demoting the only admin account.
+         * Returns TRUE when user is an administrator and there is only
+         * one administrator */
+
+        if (act_user_get_account_type (user) == ACT_USER_ACCOUNT_TYPE_STANDARD)
+                return FALSE;
+
+        if (get_num_admin (um) > 1)
+                return FALSE;
+
+        return TRUE;
+}
