@@ -2,16 +2,18 @@
 #include "config.h"
 
 #include <glib/gi18n.h>
+#include <clutter-gtk/clutter-gtk.h>
 
 #include "cc-wacom-page.h"
 #include "gsd-wacom-device.h"
+#include "clutter/clutter.h"
 
 #define FIXED_WIDTH 675
 
 void
 cc_wacom_panel_switch_to_panel (CcWacomPanel *self, const char *panel)
 {
-	g_message ("Should launch display preferences here");
+	g_message ("Should launch %s preferences here", panel);
 }
 
 static void
@@ -19,13 +21,13 @@ add_page (GList *devices,
 	  GtkWidget *notebook)
 {
 	GtkWidget *widget;
-	GsdWacomDevice *stylus, *eraser;
+	GsdWacomDevice *stylus, *eraser, *pad;
 	GList *l;
 
 	if (devices == NULL)
 		return;
 
-	stylus = eraser = NULL;
+	stylus = eraser = pad = NULL;
 	for (l = devices; l ; l = l->next) {
 		switch (gsd_wacom_device_get_device_type (l->data)) {
 		case WACOM_TYPE_ERASER:
@@ -34,6 +36,9 @@ add_page (GList *devices,
 		case WACOM_TYPE_STYLUS:
 			stylus = l->data;
 			break;
+		case WACOM_TYPE_PAD:
+			pad = l->data;
+			break;
 		default:
 			/* Nothing */
 			;
@@ -41,7 +46,7 @@ add_page (GList *devices,
 	}
 	g_list_free (devices);
 
-	widget = cc_wacom_page_new (NULL, stylus, eraser);
+	widget = cc_wacom_page_new (NULL, stylus, eraser, pad);
 	cc_wacom_page_set_navigation (CC_WACOM_PAGE (widget), GTK_NOTEBOOK (notebook), FALSE);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), widget, NULL);
 	gtk_widget_show (widget);
@@ -66,7 +71,10 @@ int main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gtk_init (&argc, &argv);
+	if (gtk_clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS) {
+		g_critical ("Unable to initialize Clutter");
+		return 1;
+        }
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
