@@ -71,21 +71,21 @@ get_casefolded_terms (char **terms)
 }
 
 static gboolean
-matches_multiple_terms (GtkTreeModel  *model,
-                        GtkTreeIter   *iter,
-                        char         **terms)
+matches_all_terms (GtkTreeModel  *model,
+                   GtkTreeIter   *iter,
+                   char         **terms)
 {
   int i;
 
   for (i = 0; terms[i]; i++)
     {
-      if (cc_shell_model_iter_matches_search (CC_SHELL_MODEL (model),
-                                              iter,
-                                              terms[i]))
-        return TRUE;
+      if (!cc_shell_model_iter_matches_search (CC_SHELL_MODEL (model),
+                                               iter,
+                                               terms[i]))
+        return FALSE;
     }
 
-  return FALSE;
+  return TRUE;
 }
 
 static GtkTreeModel *
@@ -116,8 +116,7 @@ handle_get_initial_result_set (CcShellSearchProvider2  *skeleton,
   ok = gtk_tree_model_get_iter_first (model, &iter);
   while (ok)
     {
-      if (matches_multiple_terms (model, &iter,
-                                  casefolded_terms))
+      if (matches_all_terms (model, &iter, casefolded_terms))
         {
           g_ptr_array_add (results,
                            gtk_tree_model_get_string_from_iter (model,
@@ -159,7 +158,7 @@ handle_get_subsearch_result_set (CcShellSearchProvider2  *skeleton,
     {
       if (gtk_tree_model_get_iter_from_string (model, &iter,
                                                previous_results[i]) &&
-          matches_multiple_terms (model, &iter, casefolded_terms))
+          matches_all_terms (model, &iter, casefolded_terms))
         {
           g_ptr_array_add (results, g_strdup (previous_results[i]));
         }
@@ -188,7 +187,7 @@ handle_get_result_metas (CcShellSearchProvider2  *skeleton,
   GVariantBuilder builder;
   GAppInfo *app;
   const char *id;
-  char *name, *description, *icon_string;
+  char *name, *description, *escaped_description, *icon_string;
   GIcon *icon;
 
   g_variant_builder_init (&builder, G_VARIANT_TYPE ("aa{sv}"));
@@ -206,6 +205,7 @@ handle_get_result_metas (CcShellSearchProvider2  *skeleton,
                           -1);
       id = g_app_info_get_id (app);
       icon_string = g_icon_to_string (icon);
+      escaped_description = g_markup_escape_text (description, -1);
 
       g_variant_builder_open (&builder, G_VARIANT_TYPE ("a{sv}"));
       g_variant_builder_add (&builder, "{sv}",
@@ -215,11 +215,12 @@ handle_get_result_metas (CcShellSearchProvider2  *skeleton,
       g_variant_builder_add (&builder, "{sv}",
                              "gicon", g_variant_new_string (icon_string));
       g_variant_builder_add (&builder, "{sv}",
-                             "description", g_variant_new_string (description));
+                             "description", g_variant_new_string (escaped_description));
       g_variant_builder_close (&builder);
 
       g_free (name);
       g_free (description);
+      g_free (escaped_description);
       g_free (icon_string);
       g_object_unref (app);
       g_object_unref (icon);
